#!/bin/bash

GREEN='\033[1;32m'
RED='\033[1;31m'
NC='\033[0m' # No Color

declare -a not_install_package

out() {
    echo -e "${GREEN}${1}${NC}"
    echo $(date '+%d/%m/%Y %H:%M:%S')':'${1}  >>  outlog.log
}

# Копирование настроек нового пакета, чтобы сначала настроить сервис, а потом установить пакет, который сразу запустит сервис
# Первый параметр - название нового пакета
copy_new_package_properties() {
    mkdir -p /opt/${1} && cp ./new_package_properties/${1}/application.properties /opt/${1}/application.properties
}

# Получение значения настройки
# Первый параметр - название настройки, второй - путь к файлу настроек
get_property() {
   echo `cat $2 | grep "^${1}=" | cut -d'=' -f2-`
}

get_last_property() {
    echo `cat $2 | grep "^${1}=" | tail -1 | cut -d'=' -f2-`
}

# Установить значение настройки
# Первый параметр - название настройки, второй - значение настройки, третий - путь к файлу настроек
replace_property() {
    v=`echo "${2}" | sed 's/\&/\\\&/g'`
    echo "change property ${1} to ${v} in ${3}"
    sed -i "s%$1=.*%$1=$v%g" $3
}

# Изменить значения настроек, получив сами значения из другого файла
# Первый параметр - список названий настроек, второй - путь к файлу, из которого получить значения настроек, третий - путь к файлу, в котором надо изменить значения настроек
replace_properties() {
    props=("${!1}")
    for prop_name in ${props[@]}; do
        prop_value=$(get_property "${prop_name}" "${2}")
        replace_property $prop_name $prop_value $3
    done
}

# Считать системные настройки из virgo java.conf и добавить их в файл настроек
# Первый параметр - путь к файлу настроек
copy_system_properties() {
    java_props=$(cat /opt/virgo/configuration/java.conf | grep "\-D" | sed 's/-D/-D \n/g' | sed 's/*/\\*/g')
    java_props=("${java_props[@]}" "-D")
    final_props=()
    tmp=''
    for i in ${java_props[@]}; do
        if [[ $i != *"qpid.sync_op_timeout="* && $i != *"sync_publish="* ]]; then
            if [[ $i == *'-D'* ]]; then
                if [[ $tmp != '' ]]; then
                    final_props=("${final_props[@]}" "${tmp}")
                    tmp=''
                fi
            else
                if [[ $tmp == '' ]]; then
                    tmp=$i
                else
                    tmp="${tmp} $i"
                fi
            fi
        fi
    done

    for i in "${final_props[@]}"; do
        echo "add system property $i to ${1}"
        echo "$i" | sed 's/\\\*/\*/g' >> $1
    done

}

# Изменить значения настроек для переданного мапа
# Первый параметр - мап с названиями и значениями настроек, второй - путь к файлу, в котором надо изменить значения настроек
replace_properties_by_map() {
    for prop_name in "${!1[@]}"; do
        replace_property $prop_name ${1[$prop_name]} $2
    done
}


# Cравнить дефолтные настройки с настройками у клиентов и измененные настройки пенесенсти в новый сервис
# Первый параметр - путь к файлу с дефолтными настройками, второй - путь к файлу с настройками клиента, третий - путь к файлу настроек, в который надо записать измененные настройки
add_changed_properties() {
    # это настройки подключения к бонусному серверу (используется private-office-service). Вроде этих настроек нет больше в КС
    ignore_props=("logic.server.admin.location" "logic.server.base.location" "logic.server.terminal.id")

    echo "" >> $1
    echo "" >> $2
    typeset -A default_props
    while IFS=$':= \t' read key value; do
      [[ $key = ["#"]* ]] || [[ $key = "" ]] || default_props[$key]=$value
    done < $1
    typeset -A users_props
    while IFS=$':= \t' read key value; do
      [[ $key = ["#"]* ]] || [[ $key = "" ]] || users_props[$key]=$value
    done < $2
    for k in "${!default_props[@]}"; do
        if [[ " ${ignore_props[@]} " =~ " ${k} " ]]; then
            out "ignore property ${k}"
        else
            if [[ ${default_props[$k]} != ${users_props[$k]} ]]; then
                old_value=$(get_property "${k}" "${3}")
                if [[ $old_value = "" ]]; then
                    out "add changed property $k=${users_props[$k]} to ${3}"
                    echo "$k=${users_props[$k]}" >> $3
                else
                    replace_property "${k}" "${users_props[$k]}" "${3}"
                fi
            fi
        fi
    done
}

# Установить и сконфигурировать одну из модификаций бонусного сервера
# Первый параметр - название пакета, который надо установить, второй - установлено ли mongo хранилище, третий - установлено ли mssql хранилище, четвертый - установлено ли mysql хранилище
configure_acc() {
#    typeset -A props=( ["accounting\.thrift\.admin\.port"]="7790" ["accounting\.thrift\.logic\.port"]="7791" )
    copy_new_package_properties "${1}"
    replace_property "accounting\.thrift\.admin\.port" "7790" "/opt/${1}/application.properties"
    replace_property "accounting\.thrift\.logic\.port" "7791" "/opt/${1}/application.properties"
    replace_property "server\.port" "38054" "/opt/${1}/application.properties"
    if [ "${2}" = "install ok installed" ]; then
        out "remove artix-accounting-mongo-storage"
        apt-get -y remove artix-accounting-mongo-storage
        sed -i "s%#accounting\.sql\.storage=true%accounting\.sql\.storage=false%g" "/opt/${1}/application.properties"
        sed -i "s%accounting\.sql\.type=mysql%#accounting\.sql\.type=mysql%g" "/opt/${1}/application.properties"
        sed -i "s%#accounting\.mongodb\.storage=false%accounting\.mongodb\.storage=true%g" "/opt/${1}/application.properties"
        replace_property "urls\.mongo\.database" "accounting" "/opt/${1}/application.properties"
    else
        replace_property "cashserver\.artix\.sql\.dbsuffix" "artixcsAll" "/opt/${1}/application.properties"
    fi
    if [ "${3}" = "install ok installed" ]; then
        out "remove artix-accounting-mssql-storage"
        apt-get -y remove artix-accounting-mssql-storage
        replace_property "accounting\.sql\.type" "mssql" "/opt/${1}/application.properties"
        add_changed_properties "cashserver-routing-connection-mssql.properties" "/opt/virgo/repository/usr/cashserver-routing-connection-mssql.properties" "/opt/${1}/application.properties"
    fi
    if [ "${4}" = "install ok installed" ]; then
        out "remove artix-accounting-mysql-storage"
        apt-get -y remove artix-accounting-mysql-storage
        out "install artixcs-config"
        apt-get -y install artixcs-config
        add_changed_properties "cashserver-routing-connection.properties" "/opt/virgo/repository/usr/cashserver-routing-connection.properties" "/opt/${1}/application.properties"
    fi
    add_changed_properties "accounting-logic-properties.properties" "/opt/virgo/repository/usr/accounting-logic-properties.properties" "/opt/${1}/application.properties"
    copy_system_properties "/opt/${1}/application.properties"
    out "install ${1}"
    sudo apt-get -y install ${1} || not_install_package+=(${1}) 
}



rm /etc/apt/sources.list.d/cashserver3.list
echo 'deb [arch=amd64] http://update.artix.su/cs/ cashserver4_stable cashserver4' | sudo tee /etc/apt/sources.list.d/cashserver4.list
#echo 'deb [arch=amd64] http://192.169.0.14/repository/cs/ cashserver4_testing cashserver4' | sudo tee /etc/apt/sources.list.d/cashserver4.list
apt-get update

virgoIsInstall=`dpkg-query --show --showformat='${Status}' virgo-tomcat-server`

accountingLightIsInstall=`dpkg-query --show --showformat='${Status}' artix-accounting-light`
accountingIsInstall=`dpkg-query --show --showformat='${Status}' artix-accounting`
if [ "$accountingLightIsInstall" = "install ok installed" ] || [ "$accountingIsInstall" = "install ok installed" ] ; then
    accountingMongoStorageIsInstall=`dpkg-query --show --showformat='${Status}' artix-accounting-mongo-storage`
    accountingMssqlStorageIsInstall=`dpkg-query --show --showformat='${Status}' artix-accounting-mssql-storage`
    accountingMysqlStorageIsInstall=`dpkg-query --show --showformat='${Status}' artix-accounting-mysql-storage`
    if [ "$accountingMongoStorageIsInstall" = "install ok installed" ]; then
        out "${RED}ERROR. Can't update old accounting with mongo storage to new accounting!!!!!!!!!!!!!"
        exit 10
    fi
    if [ "$accountingLightIsInstall" = "install ok installed" ]; then
        out "remove artix-accounting-light"
        apt-get -y remove artix-accounting-light
        configure_acc "artixcs-accounting-coupons" "${accountingMongoStorageIsInstall}" "${accountingMssqlStorageIsInstall}" "${accountingMysqlStorageIsInstall}"
    fi
    if [ "$accountingIsInstall" = "install ok installed" ]; then
        server_type=$(get_property "accounting.logic.serverType" "/opt/virgo/repository/usr/accounting-logic-mode.properties")
        out "remove artix-accounting"
        apt-get -y remove artix-accounting
        if [ "$server_type" = "BONUSES" ]; then
            configure_acc "artixcs-accounting-bonuses" "${accountingMongoStorageIsInstall}" "${accountingMssqlStorageIsInstall}" "${accountingMysqlStorageIsInstall}"
        fi
        if [ "$server_type" = "CERTIFICATES" ]; then
            configure_acc "artixcs-accounting-certificates" "${accountingMongoStorageIsInstall}" "${accountingMssqlStorageIsInstall}" "${accountingMysqlStorageIsInstall}"
        fi
        if [ "$server_type" = "BONUSES_CERTIFICATES" ]; then
            configure_acc "artixcs-accounting-bonuses-certificates" "${accountingMongoStorageIsInstall}" "${accountingMssqlStorageIsInstall}" "${accountingMysqlStorageIsInstall}"
        fi
    fi
fi

salesWsIsInstall=`dpkg-query --show --showformat='${Status}' artix-cashserver-sales-ws`
if [ "$salesWsIsInstall" = "install ok installed" ]; then
    out "remove artix-cashserver-sales-ws"
    sudo apt-get -y remove artix-cashserver-sales-ws
    copy_new_package_properties "artixcs-sales-ws"
    props=("urls\.create\.driver" "urls\.create\.url" "urls\.create\.postgre\.driver" "urls\.create\.postgre\.url" "connection\.connectionCount")
    out "configure artixcs-sales-ws"
    replace_properties props[@] "/opt/virgo/repository/usr/connection-properties.properties" "/opt/artixcs-sales-ws/application.properties"
    out "install artixcs-sales-ws"
    sudo apt-get -y install artixcs-sales-ws || not_install_package+=('artixcs-sales-ws') 
fi

csWebIsInstall=`dpkg-query --show --showformat='${Status}' artix-cs-web`
if [ "$csWebIsInstall" = "install ok installed" ]; then
    out "remove artix-cs-web"
    sudo apt-get -y remove artix-cs-web
    copy_new_package_properties "artixcs-web"
    java_props=`cat /opt/virgo/configuration/java.conf | grep "\-D" | sed 's/-D/\n/g'`
    for i in ${java_props[@]}; do
        if [[ $i != *"qpid.sync_op_timeout="* && $i != *"sync_publish="* ]]; then
            if [[ $i == *"ru.ssa.cashserver.web.content.tree.abstract.server.object.tree.provider.tree.elements.cache.time.for.checked.tree.provider.in.seconds"* || $i == *"su.artix.cashserver.service.cash.server.mode.invocation.handler.check.license.validate.duration"* ]]; then
                out "add system property $i to ${1}"
                echo "$i" >> /opt/artixcs-web/application.properties
            fi
        fi
    done
    out "install artixcs-web"
    sudo apt-get -y install artixcs-web || not_install_package+=('artixcs-web') 
    out "install artixcs-controlcenter"
    sudo apt-get -y install artixcs-controlcenter || not_install_package+=('artixcs-controlcenter') 
fi

dataTransferIsInstall=`dpkg-query --show --showformat='${Status}' artix-data-transfer`
if [ "$dataTransferIsInstall" = "install ok installed" ]; then
    out "remove artix-data-transfer"
    sudo apt-get -y remove artix-data-transfer
    copy_new_package_properties "artixcs-datatransfer"
    add_changed_properties "transfer-properties.properties" "/opt/virgo/repository/usr/transfer-properties.properties" "/opt/artixcs-datatransfer/application.properties"
    out "install artixcs-datatransfer"
    sudo apt-get -y install artixcs-datatransfer || not_install_package+=('artixcs-datatransfer') 
fi

privateOfficeIsInstall=`dpkg-query --show --showformat='${Status}' artix-private-office-service`
if [ "$privateOfficeIsInstall" = "install ok installed" ]; then
    out "remove artix-private-office-service"
    sudo apt-get -y remove artix-private-office-service
    copy_new_package_properties "artixcs-private-office"
    user_name=$(get_last_property "role\.private-office-service" "/opt/virgo/configuration/org.eclipse.virgo.kernel.users.properties")
    user_pass=$(get_last_property "user\.${user_name}" "/opt/virgo/configuration/org.eclipse.virgo.kernel.users.properties")
    replace_property "private\.office\.service\.user" $user_name "/opt/artixcs-private-office/application.properties"
    replace_property "private\.office\.service\.password" $user_pass "/opt/artixcs-private-office/application.properties"
    add_changed_properties "private-office-service.properties" "/opt/virgo/repository/usr/private-office-service.properties" "/opt/artixcs-private-office/application.properties"
    out "install artixcs-private-office"
    sudo apt-get -y install artixcs-private-office || not_install_package+=('artixcs-private-office') 
fi

accountingImpactsScheduler=`dpkg-query --show --showformat='${Status}' artix-accounting-impacts-scheduler`
if [ "$accountingImpactsScheduler" = "install ok installed" ]; then
    out "remove artix-accounting-impacts-scheduler"
    sudo apt-get -y remove artix-accounting-impacts-scheduler
    copy_new_package_properties "artixcs-accounting-scheduled-impacts"
    add_changed_properties "accounting-scheduled.properties" "/opt/virgo/repository/usr/accounting-scheduled.properties" "/opt/artixcs-accounting-scheduled-impacts/application.properties"
    out "install artixcs-accounting-scheduled-impacts"
    sudo apt-get -y install artixcs-accounting-scheduled-impacts || not_install_package+=('artixcs-accounting-scheduled-impacts') 
fi

processedIsInstall=`dpkg-query --show --showformat='${Status}' artix-cs-processed`
if [ "$processedIsInstall" = "install ok installed" ]; then
    out "remove artix-cs-processed"
    sudo apt-get -y remove artix-cs-processed
    out "install artixcs-processed"
    sudo apt-get -y install artixcs-processed || not_install_package+=('artixcs-processed') 
fi


# На всякий случай пометим пакеты, относящиеся к qpid-у как установленные вручную, чтобы они не удалились автоматически при удалении пакетов и при autoremove. 
# qpid еще понадобится для перемещения справочников, которые не успели прогрузиться на кассы, когда переведут режим КС на новый протокол
sudo apt-mark manual artix-qpidd-config python-qpid python-qpid-python python-qpid-qmf python-qpid-tools qpid-tools qpidd qpidd-msgstore

artixcsIsInstall=`dpkg-query --show --showformat='${Status}' artixcs`
if [ "$artixcsIsInstall" = "install ok installed" ]; then
    if dpkg -l | grep -qw artixcs-central; then
      out 'check and remove artixcs-central'
      centralIsInstalled=true
      sudo apt -y remove artixcs-central
    fi
    out 'remove artixcs'
    sudo apt-get -y remove artixcs artix-cashserver-config
    copy_new_package_properties "artixcs-rest"
    add_changed_properties "cashserver-cache.properties" "/opt/virgo/repository/usr/cashserver-cache.properties" "/opt/artixcs-rest/application.properties"
    add_changed_properties "cashserver-cashcommunication.properties" "/opt/virgo/repository/usr/cashserver-cashcommunication.properties" "/opt/artixcs-rest/application.properties"
    add_changed_properties "cashserver-core.properties" "/opt/virgo/repository/usr/cashserver-core.properties" "/opt/artixcs-rest/application.properties"
    add_changed_properties "cashserver-frontoffice.properties" "/opt/virgo/repository/usr/cashserver-frontoffice.properties" "/opt/artixcs-rest/application.properties"
    add_changed_properties "cashserver-routing-connection.properties" "/opt/virgo/repository/usr/cashserver-routing-connection.properties" "/opt/artixcs-rest/application.properties"
    add_changed_properties "cashserver-sales.properties" "/opt/virgo/repository/usr/cashserver-sales.properties" "/opt/artixcs-rest/application.properties"
    add_changed_properties "connection-properties.properties" "/opt/virgo/repository/usr/connection-properties.properties" "/opt/artixcs-rest/application.properties"
    copy_system_properties "/opt/artixcs-rest/application.properties"
    out 'install artixcs-rest'
    sudo apt-get -y install artixcs-rest artixcs-rest-lib || not_install_package+=('artixcs-rest')
    if [ "$centralIsInstalled" = true ]; then
      out 'install central if it was removed'
      sudo apt-get -y install artixcs-central
    fi

    # установим модуль загрузки продаж через grpc, чтобы после обновления на КС4 изменить на 5-ый протокол взаимодействия с кассой(загрузка продаж через grpc, отправка справочников по rest)
    out 'install artixcs-sales-loader'
    sudo apt-get -y install artixcs-sales-loader || not_install_package+=('artixcs-sales-loader') 

    # серивс для отображения информации о продажах и для отчетов
    out 'install artixcs-clickhouse-rest'
    sudo apt-get -y install artixcs-clickhouse-rest || not_install_package+=('artixcs-clickhouse-rest') 

    # установим сервис для отображения подробной информации о чеке
    out 'call script install_sale_renderer for install artixcs-sale-renderer'
    ./install_sale_renderer.sh || not_install_package+=('install_sale_renderer.sh')

#    out 'upgrade mysql'
#    sudo mysql_upgrade -u root -p
#    sudo service mysql restart
fi

out "autoremove"
apt-get -y autoremove

# пометим обратно пакеты qpid как автоустановленные
sudo apt-mark auto artix-qpidd-config python-qpid python-qpid-python python-qpid-qmf python-qpid-tools qpid-tools qpidd qpidd-msgstore

if [ "$virgoIsInstall" = "install ok installed" ]; then
    out "install artixcs-rest-routing"
    sudo apt-get -y install artixcs-rest-routing || not_install_package+=('artixcs-rest-routing') 
fi

# пишем в файлик рядом какие пакеты не установились по какой-то причине
echo ${not_install_package[@]} > not_install_package_after_script.log

#out "apt-get upgrade"
#apt-get upgrade

